@@ -40,6 +40,12 @@ DbusListType = List[DbusBasicTypes]
 DbusCompleteTypes = Union[DbusBasicTypes, DbusStructType,
                           DbusDictType, DbusVariantType, DbusListType]
 
+__STUB_ERROR = (
+    'Typing stub. You should never see this '
+    'error unless the actual module failed to load. '
+    'Check your installation.'
+)
+
 
 class SdBusSlot:
     """Holds reference to SdBus slot"""
@@ -56,7 +62,7 @@ class SdBusInterface:
         flags: int,
         callback: Callable[[SdBusMessage], Coroutine[Any, Any, None]], /
     ) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def add_property(
         self,
@@ -66,7 +72,7 @@ class SdBusInterface:
         set_function: Optional[Callable[[SdBusMessage], None]],
         flags: int, /
     ) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def add_signal(
         self,
@@ -75,90 +81,90 @@ class SdBusInterface:
         signal_args_names: Sequence[str],
         flags: int, /
     ) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
 
 class SdBusMessage:
     def append_data(self, signature: str, *args: DbusCompleteTypes) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def open_container(self, container_type: str,
                        container_signature: str, /) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def close_container(self) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def enter_container(self, container_type: str,
                         container_signature: str, /) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def exit_container(self) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def dump(self) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def seal(self) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def get_contents(self
                      ) -> Tuple[DbusCompleteTypes, ...]:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def get_member(self) -> str:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def create_reply(self) -> SdBusMessage:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def create_error_reply(
             self,
             error_name: str,
             error_message: str, /) -> SdBusMessage:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def send(self) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
-    expect_reply: bool
+    expect_reply: bool = False
 
 
 class SdBus:
     def call(self, message: SdBusMessage, /) -> SdBusMessage:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def call_async(
             self, message: SdBusMessage,
             /) -> Future[SdBusMessage]:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def drive(self) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def get_fd(self) -> int:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def new_method_call_message(
             self,
             destination_name: str, object_path: str,
             interface_name: str, member_name: str,
             /) -> SdBusMessage:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def new_property_get_message(
             self,
             destination_service_name: str, object_path: str,
             interface_name: str, member_name: str,
             /) -> SdBusMessage:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def new_property_set_message(
             self,
             destination_service_name: str, object_path: str,
             interface_name: str, member_name: str,
             /) -> SdBusMessage:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def new_signal_message(
             self,
@@ -166,11 +172,11 @@ class SdBus:
             interface_name: str,
             member_name: str,
             /) -> SdBusMessage:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def add_interface(self, new_interface: SdBusInterface,
                       object_path: str, interface_name: str, /) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def get_signal_queue_async(
         self,
@@ -178,43 +184,55 @@ class SdBus:
         interface_name: str, member_name: str,
         /
     ) -> Future[Queue[SdBusMessage]]:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def request_name_async(self, name: str, flags: int, /) -> Future[None]:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def request_name(self, name: str, flags: int, /) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def add_object_manager(self, path: str, /) -> SdBusSlot:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
     def emit_object_added(self, path: str, /) -> None:
-        ...
+        raise NotImplementedError(__STUB_ERROR)
 
 
 def sd_bus_open() -> SdBus:
-    return SdBus()
+    raise NotImplementedError(__STUB_ERROR)
 
 
 def sd_bus_open_user() -> SdBus:
-    return SdBus()
+    raise NotImplementedError(__STUB_ERROR)
 
 
 def sd_bus_open_system() -> SdBus:
-    return SdBus()
+    raise NotImplementedError(__STUB_ERROR)
+
+
+def sd_bus_open_system_remote(host: str, /) -> SdBus:
+    raise NotImplementedError(__STUB_ERROR)
+
+
+def sd_bus_open_user_machine(machine: str, /) -> SdBus:
+    raise NotImplementedError(__STUB_ERROR)
+
+
+def sd_bus_open_system_machine(machine: str, /) -> SdBus:
+    raise NotImplementedError(__STUB_ERROR)
 
 
 def encode_object_path(prefix: str, external: str) -> str:
-    return prefix + external
+    raise NotImplementedError(__STUB_ERROR)
 
 
 def decode_object_path(prefix: str, full_path: str) -> str:
-    return full_path
+    raise NotImplementedError(__STUB_ERROR)
 
 
 def add_exception_mapping(exc: Exception, /) -> None:
-    ...
+    ...  # We want to be able to generate docs without module
 
 
 class SdBusBaseError(Exception):
