@@ -19,11 +19,9 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 from __future__ import annotations
 
-from os import environ
-from unittest import IsolatedAsyncioTestCase, SkipTest, main
+from unittest import main
 
-from sdbus import DbusServiceUnknownError
-from sdbus.async_proxies import FreedesktopDbus, FreedesktopNotifications
+from sdbus_async.dbus_daemon import FreedesktopDbus
 
 from .common_test_util import TempDbusTest
 
@@ -39,18 +37,5 @@ class TestFreedesktopDbus(TempDbusTest):
         self.assertIsInstance(await dbus_object.features, list)
 
 
-class TestFreedesktopNotifications(IsolatedAsyncioTestCase):
-
-    async def test_capabilities(self) -> None:
-        if 'DBUS_SESSION_BUS_ADDRESS' not in environ:
-            raise SkipTest('No session dbus running')
-
-        notifications = FreedesktopNotifications()
-        try:
-            self.assertIsInstance(await notifications.get_capabilities(), list)
-        except DbusServiceUnknownError:
-            raise SkipTest('No notifications daemon running')
-
-
 if __name__ == "__main__":
     main()
